==> retro_data_structures-0.26.0-py3-none-any.whl: retro_data_structures/properties/echoes/archetypes/TweakTargeting_VulnerabilityIndicator.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection
from retro_data_structures.properties.echoes.core.Color import Color

if typing.TYPE_CHECKING:
    class TweakTargeting_VulnerabilityIndicatorJson(typing_extensions.TypedDict):
        indicator_angle: float
        indicator_color: json_util.JsonValue
    

_FAST_FORMAT = None
_FAST_IDS = (0x4c86af42, 0x436b66ac)


@dataclasses.dataclass()
class TweakTargeting_VulnerabilityIndicator(BaseProperty):
    indicator_angle: float = dataclasses.field(default=0.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x4c86af42, original_name='IndicatorAngle'
        ),
    })
    indicator_color: Color = dataclasses.field(default_factory=lambda: Color(r=0.0, g=0.0, b=0.0, a=0.0), metadata={
        'reflection': FieldReflection[Color](
            Color, id=0x436b66ac, original_name='IndicatorColor', from_json=Color.from_json, to_json=Color.to_json
        ),
    })

    @classmethod
    def game(cls) -> Game:
        return Game.ECHOES

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 2:
            return None
    
        global _FAST_FORMAT
        if _FAST_FORMAT is None:
            _FAST_FORMAT = struct.Struct('>LHfLHffff')
    
        dec = _FAST_FORMAT.unpack(data.read(32))
        assert (dec[0], dec[3]) == _FAST_IDS
        return cls(
            dec[2],
            Color(*dec[5:9]),
        )

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x02')  # 2 properties

        data.write(b'L\x86\xafB')  # 0x4c86af42
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.indicator_angle))

        data.write(b'Ckf\xac')  # 0x436b66ac
        data.write(b'\x00\x10')  # size
        self.indicator_color.to_stream(data)

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("TweakTargeting_VulnerabilityIndicatorJson", data)
        return cls(
            indicator_angle=json_data['indicator_angle'],
            indicator_color=Color.from_json(json_data['indicator_color']),
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'indicator_angle': self.indicator_angle,
            'indicator_color': self.indicator_color.to_json(),
        }

    def dependencies_for(self, asset_manager):
        yield from []


def _decode_indicator_angle(data: typing.BinaryIO, property_size: int):
    return struct.unpack('>f', data.read(4))[0]


def _decode_indicator_color(data: typing.BinaryIO, property_size: int):
    return Color.from_stream(data)


_property_decoder: typing.Dict[int, typing.Tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0x4c86af42: ('indicator_angle', _decode_indicator_angle),
    0x436b66ac: ('indicator_color', _decode_indicator_color),
}

==> retro_data_structures-0.29.0-py3-none-any.whl: retro_data_structures/properties/echoes/archetypes/TweakTargeting_VulnerabilityIndicator.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection
from retro_data_structures.properties.echoes.core.Color import Color

if typing.TYPE_CHECKING:
    from retro_data_structures.asset_manager import AssetManager
    from retro_data_structures.base_resource import Dependency

    class TweakTargeting_VulnerabilityIndicatorJson(typing_extensions.TypedDict):
        indicator_angle: float
        indicator_color: json_util.JsonValue
    

_FAST_FORMAT: struct.Struct | None = None
_FAST_IDS = (0x4c86af42, 0x436b66ac)


@dataclasses.dataclass()
class TweakTargeting_VulnerabilityIndicator(BaseProperty):
    indicator_angle: float = dataclasses.field(default=0.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x4c86af42, original_name='IndicatorAngle'
        ),
    })
    indicator_color: Color = dataclasses.field(default_factory=lambda: Color(r=0.0, g=0.0, b=0.0, a=0.0), metadata={
        'reflection': FieldReflection[Color](
            Color, id=0x436b66ac, original_name='IndicatorColor', from_json=Color.from_json, to_json=Color.to_json
        ),
    })

    @classmethod
    def game(cls) -> Game:
        return Game.ECHOES

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 2:
            return None
    
        global _FAST_FORMAT
        if _FAST_FORMAT is None:
            _FAST_FORMAT = struct.Struct('>LHfLHffff')
    
        dec = _FAST_FORMAT.unpack(data.read(32))
        assert (dec[0], dec[3]) == _FAST_IDS
        return cls(
            dec[2],
            Color(*dec[5:9]),
        )

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x02')  # 2 properties

        data.write(b'L\x86\xafB')  # 0x4c86af42
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.indicator_angle))

        data.write(b'Ckf\xac')  # 0x436b66ac
        data.write(b'\x00\x10')  # size
        self.indicator_color.to_stream(data)

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("TweakTargeting_VulnerabilityIndicatorJson", data)
        return cls(
            indicator_angle=json_data['indicator_angle'],
            indicator_color=Color.from_json(json_data['indicator_color']),
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'indicator_angle': self.indicator_angle,
            'indicator_color': self.indicator_color.to_json(),
        }

    def dependencies_for(self, asset_manager: AssetManager) -> typing.Iterator[Dependency]:
        yield from []


def _decode_indicator_angle(data: typing.BinaryIO, property_size: int) -> float:
    return struct.unpack('>f', data.read(4))[0]


def _decode_indicator_color(data: typing.BinaryIO, property_size: int) -> Color:
    return Color.from_stream(data)


_property_decoder: dict[int, tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0x4c86af42: ('indicator_angle', _decode_indicator_angle),
    0x436b66ac: ('indicator_color', _decode_indicator_color),
}
